"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createWrappedCoin = exports.createWrappedCoinType = exports.transferTokens = exports.registerChain = exports.deployCoin = exports.completeTransferWithPayload = exports.completeTransferAndRegister = exports.completeTransfer = exports.attestToken = void 0;
var algorand_1 = require("../../algorand");
var utils_1 = require("../../utils");
// Attest token
var attestToken = function (tokenBridgeAddress, tokenChain, tokenAddress) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var assetType = utils_1.getAssetFullyQualifiedType(tokenBridgeAddress, utils_1.coalesceChainId(tokenChain), tokenAddress);
    if (!assetType)
        throw "Invalid asset address.";
    return {
        function: tokenBridgeAddress + "::attest_token::attest_token_entry",
        type_arguments: [assetType],
        arguments: [],
    };
};
exports.attestToken = attestToken;
// Complete transfer
var completeTransfer = function (tokenBridgeAddress, transferVAA, feeRecipient) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var parsedVAA = algorand_1._parseVAAAlgorand(transferVAA);
    if (!parsedVAA.FromChain || !parsedVAA.Contract || !parsedVAA.ToChain) {
        throw "VAA does not contain required information";
    }
    if (parsedVAA.ToChain !== utils_1.CHAIN_ID_APTOS) {
        throw "Transfer is not destined for Aptos";
    }
    utils_1.assertChain(parsedVAA.FromChain);
    var assetType = utils_1.getAssetFullyQualifiedType(tokenBridgeAddress, utils_1.coalesceChainId(parsedVAA.FromChain), parsedVAA.Contract);
    if (!assetType)
        throw "Invalid asset address.";
    return {
        function: tokenBridgeAddress + "::complete_transfer::submit_vaa_entry",
        type_arguments: [assetType],
        arguments: [transferVAA, feeRecipient],
    };
};
exports.completeTransfer = completeTransfer;
var completeTransferAndRegister = function (tokenBridgeAddress, transferVAA) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var parsedVAA = algorand_1._parseVAAAlgorand(transferVAA);
    if (!parsedVAA.FromChain || !parsedVAA.Contract || !parsedVAA.ToChain) {
        throw "VAA does not contain required information";
    }
    if (parsedVAA.ToChain !== utils_1.CHAIN_ID_APTOS) {
        throw "Transfer is not destined for Aptos";
    }
    utils_1.assertChain(parsedVAA.FromChain);
    var assetType = utils_1.getAssetFullyQualifiedType(tokenBridgeAddress, utils_1.coalesceChainId(parsedVAA.FromChain), parsedVAA.Contract);
    if (!assetType)
        throw "Invalid asset address.";
    return {
        function: tokenBridgeAddress + "::complete_transfer::submit_vaa_and_register_entry",
        type_arguments: [assetType],
        arguments: [transferVAA],
    };
};
exports.completeTransferAndRegister = completeTransferAndRegister;
var completeTransferWithPayload = function (_tokenBridgeAddress, _tokenChain, _tokenAddress, _vaa) {
    throw new Error("Completing transfers with payload is not yet supported in the sdk");
};
exports.completeTransferWithPayload = completeTransferWithPayload;
// Deploy coin
// don't need `signer` and `&signer` in argument list because the Move VM will inject them
var deployCoin = function (tokenBridgeAddress) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    return {
        function: tokenBridgeAddress + "::deploy_coin::deploy_coin",
        type_arguments: [],
        arguments: [],
    };
};
exports.deployCoin = deployCoin;
// Register chain
var registerChain = function (tokenBridgeAddress, vaa) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    return {
        function: tokenBridgeAddress + "::register_chain::submit_vaa_entry",
        type_arguments: [],
        arguments: [vaa],
    };
};
exports.registerChain = registerChain;
// Transfer tokens
var transferTokens = function (tokenBridgeAddress, tokenChain, tokenAddress, amount, recipientChain, recipient, relayerFee, wormholeFee, nonce, payload) {
    if (payload === void 0) { payload = ""; }
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var assetType = utils_1.getAssetFullyQualifiedType(tokenBridgeAddress, utils_1.coalesceChainId(tokenChain), tokenAddress);
    if (!assetType)
        throw "Invalid asset address.";
    var recipientChainId = utils_1.coalesceChainId(recipientChain);
    if (payload) {
        throw new Error("Transfer with payload are not yet supported in the sdk");
    }
    else {
        return {
            function: tokenBridgeAddress + "::transfer_tokens::transfer_tokens_entry",
            type_arguments: [assetType],
            arguments: [amount, recipientChainId, recipient, relayerFee, wormholeFee, nonce],
        };
    }
};
exports.transferTokens = transferTokens;
// Created wrapped coin
var createWrappedCoinType = function (tokenBridgeAddress, vaa) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    return {
        function: tokenBridgeAddress + "::wrapped::create_wrapped_coin_type",
        type_arguments: [],
        arguments: [vaa],
    };
};
exports.createWrappedCoinType = createWrappedCoinType;
var createWrappedCoin = function (tokenBridgeAddress, attestVAA) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var parsedVAA = algorand_1._parseVAAAlgorand(attestVAA);
    if (!parsedVAA.FromChain || !parsedVAA.Contract) {
        throw "VAA does not contain required information";
    }
    utils_1.assertChain(parsedVAA.FromChain);
    var assetType = utils_1.getAssetFullyQualifiedType(tokenBridgeAddress, utils_1.coalesceChainId(parsedVAA.FromChain), parsedVAA.Contract);
    if (!assetType)
        throw "Invalid asset address.";
    return {
        function: tokenBridgeAddress + "::wrapped::create_wrapped_coin",
        type_arguments: [assetType],
        arguments: [attestVAA],
    };
};
exports.createWrappedCoin = createWrappedCoin;
