import { _parseVAAAlgorand } from "../../algorand";
import { assertChain, CHAIN_ID_APTOS, coalesceChainId, getAssetFullyQualifiedType } from "../../utils";
// Attest token
export var attestToken = function (tokenBridgeAddress, tokenChain, tokenAddress) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var assetType = getAssetFullyQualifiedType(tokenBridgeAddress, coalesceChainId(tokenChain), tokenAddress);
    if (!assetType)
        throw "Invalid asset address.";
    return {
        function: tokenBridgeAddress + "::attest_token::attest_token_entry",
        type_arguments: [assetType],
        arguments: [],
    };
};
// Complete transfer
export var completeTransfer = function (tokenBridgeAddress, transferVAA, feeRecipient) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var parsedVAA = _parseVAAAlgorand(transferVAA);
    if (!parsedVAA.FromChain || !parsedVAA.Contract || !parsedVAA.ToChain) {
        throw "VAA does not contain required information";
    }
    if (parsedVAA.ToChain !== CHAIN_ID_APTOS) {
        throw "Transfer is not destined for Aptos";
    }
    assertChain(parsedVAA.FromChain);
    var assetType = getAssetFullyQualifiedType(tokenBridgeAddress, coalesceChainId(parsedVAA.FromChain), parsedVAA.Contract);
    if (!assetType)
        throw "Invalid asset address.";
    return {
        function: tokenBridgeAddress + "::complete_transfer::submit_vaa_entry",
        type_arguments: [assetType],
        arguments: [transferVAA, feeRecipient],
    };
};
export var completeTransferAndRegister = function (tokenBridgeAddress, transferVAA) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var parsedVAA = _parseVAAAlgorand(transferVAA);
    if (!parsedVAA.FromChain || !parsedVAA.Contract || !parsedVAA.ToChain) {
        throw "VAA does not contain required information";
    }
    if (parsedVAA.ToChain !== CHAIN_ID_APTOS) {
        throw "Transfer is not destined for Aptos";
    }
    assertChain(parsedVAA.FromChain);
    var assetType = getAssetFullyQualifiedType(tokenBridgeAddress, coalesceChainId(parsedVAA.FromChain), parsedVAA.Contract);
    if (!assetType)
        throw "Invalid asset address.";
    return {
        function: tokenBridgeAddress + "::complete_transfer::submit_vaa_and_register_entry",
        type_arguments: [assetType],
        arguments: [transferVAA],
    };
};
export var completeTransferWithPayload = function (_tokenBridgeAddress, _tokenChain, _tokenAddress, _vaa) {
    throw new Error("Completing transfers with payload is not yet supported in the sdk");
};
// Deploy coin
// don't need `signer` and `&signer` in argument list because the Move VM will inject them
export var deployCoin = function (tokenBridgeAddress) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    return {
        function: tokenBridgeAddress + "::deploy_coin::deploy_coin",
        type_arguments: [],
        arguments: [],
    };
};
// Register chain
export var registerChain = function (tokenBridgeAddress, vaa) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    return {
        function: tokenBridgeAddress + "::register_chain::submit_vaa_entry",
        type_arguments: [],
        arguments: [vaa],
    };
};
// Transfer tokens
export var transferTokens = function (tokenBridgeAddress, tokenChain, tokenAddress, amount, recipientChain, recipient, relayerFee, wormholeFee, nonce, payload) {
    if (payload === void 0) { payload = ""; }
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var assetType = getAssetFullyQualifiedType(tokenBridgeAddress, coalesceChainId(tokenChain), tokenAddress);
    if (!assetType)
        throw "Invalid asset address.";
    var recipientChainId = coalesceChainId(recipientChain);
    if (payload) {
        throw new Error("Transfer with payload are not yet supported in the sdk");
    }
    else {
        return {
            function: tokenBridgeAddress + "::transfer_tokens::transfer_tokens_entry",
            type_arguments: [assetType],
            arguments: [amount, recipientChainId, recipient, relayerFee, wormholeFee, nonce],
        };
    }
};
// Created wrapped coin
export var createWrappedCoinType = function (tokenBridgeAddress, vaa) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    return {
        function: tokenBridgeAddress + "::wrapped::create_wrapped_coin_type",
        type_arguments: [],
        arguments: [vaa],
    };
};
export var createWrappedCoin = function (tokenBridgeAddress, attestVAA) {
    if (!tokenBridgeAddress)
        throw "Need token bridge address.";
    var parsedVAA = _parseVAAAlgorand(attestVAA);
    if (!parsedVAA.FromChain || !parsedVAA.Contract) {
        throw "VAA does not contain required information";
    }
    assertChain(parsedVAA.FromChain);
    var assetType = getAssetFullyQualifiedType(tokenBridgeAddress, coalesceChainId(parsedVAA.FromChain), parsedVAA.Contract);
    if (!assetType)
        throw "Invalid asset address.";
    return {
        function: tokenBridgeAddress + "::wrapped::create_wrapped_coin",
        type_arguments: [assetType],
        arguments: [attestVAA],
    };
};
